%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% "THE BEER-WARE LICENSE" (Revision 42):
% Mario Hellmich <mario.hellmich@web.de> wrote this file. As long as you retain
% this notice you can do whatever you want with this stuff. If we meet some
% day, and you think this stuff is worth it, you can buy me a beer in return.
% Mario Hellmich
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% File: nfmeas.m
% 
% Version: 1.1
% Created: 31.01.2023
% 
% Author: Mario Hellmich <mario.hellmich@web.de>
% Project: Noise Figure Measurements with a Spectrum Analyzer
% 
% Summary: This file implements a GNU Octave function which calculates noise
% factor and gain of a DUT from frequency dependent narrow band level 
% measurements with a calibrated noise source with given ENR table. The 
% measurement uses the Y-factor method.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function varargout = nfmeas(enrtable, freq, dut_h, dut_c, varargin)
	dut_h = reshape(dut_h, [], 1);												% Resize arguments
	dut_c = reshape(dut_c, [], 1);
	freq  = reshape(freq, [], 1);
	
	if( (length(freq)!=length(dut_h)) || (length(freq)!=length(dut_c)) )		% Check if arguments are of equal length
		error("'nfmeas': unequal length input vectors");
	endif
	
	ENR    = interp1(enrtable(1,:), enrtable(2,:), freq', "cubic")';			% Frequency dependent ENR, interpolated from ENR table
	Y_tot  = dut_h ./ dut_c;													% Total Y-factor of measurement system and DUT
	
	if(nargin == 4)
		NF_tot = ENR ./ (Y_tot - 1);											% Total noise factor of measurement system and DUT
		varargout = {NF_tot};													% Return
	elseif(nargin == 8)
		cal_h     = varargin{1};												% Calibration measurement with noise source on (hot)
		cal_c     = varargin{2};												% Calibration measurement with noise source off (cold)
		T_ref     = varargin{3};												% Reference temperature according to which the noise source ENR is specified (in Kelvin)
		T_act     = varargin{4};												% Actual temperature of noise source during measurement (in Kelvin)
		
		cal_h = reshape(cal_h, [], 1);											% Resize arguments
		cal_c = reshape(cal_c, [], 1);
		
		if( (length(freq)!=length(cal_h)) || (length(freq)!=length(cal_c)) )	% Check if arguments are of equal length
			error("'nfmeas': unequal length input vectors");
		endif
		if( (isscalar(T_ref) == 0) || (isscalar(T_act) == 0) )					% Check if temperature arguments are scalars
			error("'nfmeas': Reference and actual temperature must be scalars");
		endif
		
		NF_tot    = (ENR .* (T_act / T_ref)) ./ (Y_tot - 1);					% Total temperature corrected noise factor of measurement system and DUT
		Y_cal     = cal_h ./ cal_c;												% Y-factor of measurement system
		NF_cal    = (ENR .* (T_act / T_ref)) ./ (Y_cal - 1);					% Noise factor of measurement system
		gain_dut  = (dut_h - dut_c) ./ (cal_h - cal_c);							% DUT gain
		NF_dut    = NF_tot - ((NF_cal - 1) ./ gain_dut);						% Noise factor of DUT corrected for measurement system noise (according to Friis formula)
		
		varargout = {NF_dut, gain_dut};											% Return
	else
		error("'nfmeas': incorrect number of arguments.\n\
		       When calibration measurement data is specified, then also\n\
			   the noise source ENR reference temperature and the actual\n\
			   temperature during measurement must be specified.");
	endif
endfunction